function S = f2v(v,f)

% Compute the face to vertex interpolation matrix.
%
% If you use this code in your own work, please cite the following paper:
% [1] P. T. Choi and L. M. Lui, 
%     "Fast Disk Conformal Parameterization of Simply-Connected Open Surfaces."
%     Journal of Scientific Computing, 65(3), pp. 1065-1090, 2015.
%
% Copyright (c) 2014-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

    ring = vertexAttachments(TriRep(f,v));
    nv = length(v); nf = length(f);
    II = cellfun(@times,ring,num2cell(zeros(nv,1)),'UniformOutput',0);
    II = cell2mat(cellfun(@plus,II,num2cell(1:nv)','UniformOutput',0)')';
    JJ = cell2mat(ring')';
    avg = cellfun(@length,ring);
    S = sparse(II,JJ,ones(length(JJ),1),nv,nf);
    S = sparse(1:nv,1:nv,1./avg)*S;

end